-- with trace
-- Termometer based upon a Dallas DS1620 chip via parallel port on the PC
-- From an article appeared at Electronique Practique

-- By Jesus Consuegra (jconsuegra@redestb.es)

-- Acknowledgments to Robert Craig for such a nice environment,
--      to Jiri Babor for the font procedures, 
--      to Jacques Deschemes for his help in delay.

-- Version 1.1 Adapted to the new bit functions in Euphoria 1.5
-- Corrected a bug that did't raise properly power line.

include udelay.e        -- Sligth modification of delay.e. Same but the
			-- parameter is in milliseconds
include ports.e         
include keypress.e      -- Missing in get.e. Keypress() return true if
			-- a key has been pressed, but doesn't get the
			-- character. (This one is mine!)
include graphics.e      
include font.e          
include file.e


-- Globals --

byte PortCommand

constant PowerOn    = #01,  -- We power the termometer on by raising D0
	 PowerOff   = #FE,
	 ClockOn    = #20,
	 ClockOff   = #DF,
	 ResetOn    = #BF,
	 ResetOff   = #40,
	 OutHigh    = #80,
	 OutLow     = #7F,
	 OutPort    = #378,     -- Change this if you are using LPT2 or 3
	 InPort     = OutPort + 1
-- ---------------------------------------       
procedure InitPort()
    PortCommand = PowerOn
    Output(PortCommand, OutPort)
    uDelay(1000)
end procedure
-- ---------------------------------------
procedure ClosePort()
    PortCommand = PowerOff
    Output(PortCommand, OutPort)
    uDelay(1000)
end procedure
-- ---------------------------------------
procedure SendControl(byte ControlData)
byte Control
Control = ControlData
for i=0  to 7 do
    PortCommand = and_bits(PortCommand, ClockOff)
    Output(PortCommand, OutPort)
    if (remainder(Control,2)=0) then
	PortCommand = and_bits(PortCommand, OutLow)
    else
	PortCommand = or_bits(PortCommand, OutHigh)
    end if
    Output(PortCommand, OutPort)
    Control = floor (Control / 2)
    uDelay(5)
    PortCommand = or_bits(PortCommand, ClockOn)
    Output(PortCommand, OutPort)
    uDelay(5)
end for
end procedure
-- --------------------------------------
procedure Reset()
    PortCommand = and_bits(PortCommand, ResetOn)
    Output(PortCommand, OutPort)
    uDelay(5)
    PortCommand = or_bits(PortCommand, ResetOff)
    Output(PortCommand, OutPort)
    uDelay(5)
end procedure
-- --------------------------------------
function ReadChip()
integer DataIn
byte    temp
--  PortCommand = 0
    Reset()
    SendControl(#0C)
    SendControl(#03)
    Reset()
    SendControl(#EE)
    SendControl(#00)
    Reset()
    SendControl(#AA)
    PortCommand = or_bits(PortCommand, OutHigh)
    Output(PortCommand, OutPort)
    DataIn = 0
    for i=0  to 8 do
	PortCommand = and_bits(PortCommand, ClockOff)
	Output(PortCommand, OutPort)
	uDelay(5)
	temp = Input(InPort)
	DataIn = DataIn + (and_bits(floor(temp/64), #01) * power(2,i))
	PortCommand = or_bits(PortCommand, ClockOn)
	Output(PortCommand, OutPort)
	uDelay(5)
    end for
    return DataIn
end function
-- --------------------------------------
--
-- Main body
--
-- --------------------------------------

integer Temp1
atom    TempE
sequence f

integer Xt,Yt,X0,Y0,
	Xr, Yr

InitPort()
if graphics_mode(18) then
end if 

    X0 = 0
    Y0 = 0
    Xt = 276
    Yt = 46
    set_rand(12345)

    fy  = 10  
    fsc = 0
    ftc = 14    
    f   ={}
    f   = f&fload("Modern28.f")
    fx  = 10
    
while (not keypressed()) do
    Temp1 = ReadChip()
    if (Temp1 >= 256) then
	Temp1 = Temp1 - 512
    end if
    TempE = Temp1/2
    Xr = rand(639-Xt)
    Yr = rand(479-Yt)
    polygon(BLACK,1,{{0,0},{640,0},{640,480},{0,480}})
    polygon(GREEN,0,{{X0+Xr,Y0+Yr},{Xt+Xr,Y0+Yr},
		    {Xt+Xr,Yt+Yr},{X0+Xr,Yt+Yr}})
    fx = Xr + 10
    fy = Yr + 10
    fprint(sprintf("Centigrade temperature is: %-5.1f", TempE))
    fp[1] = 0
    fp[6] = 0
    fx=10
    fy=10
    ftc=14
    uDelay(1000)
end while
ClosePort()
if graphics_mode(-1) then
end if
-- Freeports()
